#' pfclust: Power Fuzzy Clustering and Cluster-wise Regression
#'
#' Implements Power Fuzzy Clustering (PFC) and Power Fuzzy Cluster-wise
#' Regression (PFCR) with Minkowski and adaptive regularised Mahalanobis
#' distances.
#'
#' The two main user-facing functions are:
#' \itemize{
#'   \item [PFC()] — fuzzy clustering (no covariates)
#'   \item [PFCR()] — fuzzy cluster-wise linear regression
#' }
#'
#' @keywords internal
"_PACKAGE"
