% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pfclust-package.R
\docType{package}
\name{pfclust-package}
\alias{pfclust}
\alias{pfclust-package}
\title{pfclust: Power Fuzzy Clustering and Cluster-wise Regression}
\description{
Implements Power Fuzzy Clustering (PFC) and Power Fuzzy Cluster-wise
Regression (PFCR) with Minkowski and adaptive regularised Mahalanobis
distances.
}
\details{
The two main user-facing functions are:
\itemize{
\item \code{\link[=PFC]{PFC()}} — fuzzy clustering (no covariates)
\item \code{\link[=PFCR]{PFCR()}} — fuzzy cluster-wise linear regression
}
}
\author{
\strong{Maintainer}: Phuc Thinh Nguyen \email{phucthinh010603@yahoo.com}

Authors:
\itemize{
  \item Cristina Tortora \email{cristina.tortora@sjsu.edu} [thesis advisor, degree supervisor]
  \item Antonio Punzo \email{antonio.punzo@unict.it} [thesis advisor, degree supervisor]
}

}
\keyword{internal}
